/**
 * Menu string definitions
 */

#pragma codeseg XMENUSTR
#pragma constseg XMENUSTR

#include "dpf.h"
#include "utils.h"
#include "print.h"
#include "global.h"
#include "xmenu.h"
#include "xmenuhook.h"

#define _RESOLVE(x) #x
#define STRING(x) _RESOLVE(x)

void show_testpattern(void);

static __code const char * __code xmenu_strings[] = {
/* -- XSTR_LCD -- */
//#ifdef LCD_CONTROLLER_CUSTOM
	"Unknown (custom)", 0,
//#else
//	STRING(_INTERNAL_TAG) " (compatible)", 0,
//#endif

/* -- XSTR_CREDITS -- */
#if NUM_COLS_LARGE > 16
	" By hackfin@section5.ch", "     and superelchi",
#else
	"By hackfin &", "    superelchi",
#endif

/* -- XSTR_VERSION -- */
	STRING(BUILD_ID), 0,

/* -- XSTR_WWW -- */
	"http://dpf-ax.sourceforge.net", 0,

/* -- XSTR_REBOOT -- */
#if NUM_COLS_LARGE > 16
	"Changes will be applied", " on next reset/reboot",
#else
	"Reboot/reset", "to apply",
#endif
};

#define NUM_XSTRINGS ((sizeof(xmenu_strings) / sizeof(__code char *)) / 2)

void put_xstring(unsigned char id) __banked
{
#ifdef DEBUG_DEV		
	unsigned char i, c;
#endif
	if (id && id <= NUM_XSTRINGS)
	{
		id--;
		id *= 2;
		puts(xmenu_strings[id]);
		if (xmenu_strings[id+1])
		{
			puts("\n");
			puts(xmenu_strings[id+1]);
		}
	}
	else
	{	
	// Dynamically build strings
		if (id == XHOOK_FLASH)
		{
			flash_detect();
			flash_print_size();
		}
#ifdef DEBUG_DEV		
		else if (id == XHOOK_DEBUG)
		{
			for (i = 0; i < g_debugcount; i++)
			{
				c = g_debugbuf[i];
				putchar(c & 0x7F);
				if (c & 0x80)
				{
				    i++;
				    putchar('[');
				    print_short(*((unsigned short *) &g_debugbuf[i++]));
				    putchar(']');
			        }
				
			}
		}
#endif
		else if (id == XHOOK_TESTPATTERN)
		{
			show_testpattern();
		}
	}
}

#if RESOL_X == 240	// 240x320
#define GREY_VSTARTX1     85
#define GREY_VSTARTY1     25
#define GREY_VDX          0
#define GREY_VDY          18
#define GREY_VWIDTH       35
#define GREY_VHEIGHT      18
#define GREY_VSTARTX2     GREY_VSTARTX1 + GREY_VWIDTH
#define GREY_VSTARTY2     GREY_VSTARTY1

#define GREY_HSTARTX1     20
#define GREY_HSTARTY1     235
#define GREY_HDX          18
#define GREY_HDY          0
#define GREY_HWIDTH       18
#define GREY_HHEIGHT      30
#define GREY_HSTARTX2     GREY_HSTARTX1
#define GREY_HSTARTY2     GREY_HSTARTY1 + GREY_HHEIGHT

#define COLOR_STARTX1     20
#define COLOR_STARTY1     25
#define COLOR_DX          0
#define COLOR_DY          49
#define COLOR_WIDTH       49
#define COLOR_HEIGHT      COLOR_DY
#define COLOR_STARTX2     GREY_VSTARTX2 + GREY_VWIDTH + 15
#define COLOR_STARTY2     COLOR_STARTY1
#else
#if RESOL_X == 320	// 320x240
#define GREY_VSTARTX1     25
#define GREY_VSTARTY1     85
#define GREY_VDX          18
#define GREY_VDY          0
#define GREY_VWIDTH       18
#define GREY_VHEIGHT      35
#define GREY_VSTARTX2     GREY_VSTARTX1
#define GREY_VSTARTY2     GREY_VSTARTY1 + GREY_VHEIGHT

#define GREY_HSTARTX1     235
#define GREY_HSTARTY1     20
#define GREY_HDX          0
#define GREY_HDY          18
#define GREY_HWIDTH       30
#define GREY_HHEIGHT      18
#define GREY_HSTARTX2     GREY_HSTARTX1 + GREY_HWIDTH
#define GREY_HSTARTY2     GREY_HSTARTY1

#define COLOR_STARTX1     25
#define COLOR_STARTY1     20
#define COLOR_DX          49
#define COLOR_DY          0
#define COLOR_WIDTH       COLOR_DX
#define COLOR_HEIGHT      49
#define COLOR_STARTX2     COLOR_STARTX1
#define COLOR_STARTY2     GREY_VSTARTY2 + GREY_VHEIGHT + 15
#else
			// assume 128x128
#define GREY_VSTARTX1     46
#define GREY_VSTARTY1     14
#define GREY_VDX          0
#define GREY_VDY          9
#define GREY_VWIDTH       18
#define GREY_VHEIGHT      9
#define GREY_VSTARTX2     GREY_VSTARTX1 + GREY_VWIDTH
#define GREY_VSTARTY2     GREY_VSTARTY1

#define COLOR_STARTX1     15
#define COLOR_STARTY1     14
#define COLOR_DX          0
#define COLOR_DY          25
#define COLOR_WIDTH       22
#define COLOR_HEIGHT      COLOR_DY
#define COLOR_STARTX2     GREY_VSTARTX2 + GREY_VWIDTH + 9
#define COLOR_STARTY2     COLOR_STARTY1
#endif
#endif

typedef struct {
	unsigned int xs, xe;
	unsigned int ys, ye;
	unsigned int color;
	unsigned int len;
} SWATCH;


const __code SWATCH swatches[] = {
  // Frame
  { 0, RESOL_X - 1, 0, 0, RGB565(255, 255, 255), RESOL_X },
  { 0, RESOL_X - 1, RESOL_Y - 1, RESOL_Y - 1, RGB565(255, 255, 255), RESOL_X },
  { 0, 0, 0, RESOL_Y - 1, RGB565(255, 255, 255), RESOL_Y },
  { RESOL_X - 1, RESOL_X - 1, 0, RESOL_Y - 1, RGB565(255, 255, 255), RESOL_Y },

  { 2, RESOL_X - 2 - 1, 2, 2, RGB565(255, 255, 255), RESOL_X - 4 },
  { 2, RESOL_X - 2 - 1, RESOL_Y - 2 - 1, RESOL_Y - 2 - 1, RGB565(255, 255, 255), RESOL_X - 4 },
  { 2, 2, 2, RESOL_Y - 2 - 1, RGB565(255, 255, 255), RESOL_Y - 4},
  { RESOL_X - 2 - 1, RESOL_X - 2 - 1, 2, RESOL_Y - 2 - 1, RGB565(255, 255, 255), RESOL_Y - 4},

  { 4, RESOL_X - 4 - 1, 4, 4, RGB565(255, 255, 255), RESOL_X - 8 },
  { 4, RESOL_X - 4 - 1, RESOL_Y - 4 - 1, RESOL_Y - 4 - 1, RGB565(255, 255, 255), RESOL_X - 8 },
  { 4, 4, 4, RESOL_Y - 4 - 1, RGB565(255, 255, 255), RESOL_Y - 8},
  { RESOL_X - 4 - 1, RESOL_X - 4 - 1, 4, RESOL_Y - 4 - 1, RGB565(255, 255, 255), RESOL_Y - 8},

  { 6, RESOL_X - 6 - 1, 6, 6, RGB565(255, 255, 255), RESOL_X - 12 },
  { 6, RESOL_X - 6 - 1, RESOL_Y - 6 - 1, RESOL_Y - 6 - 1, RGB565(255, 255, 255), RESOL_X - 12 },
  { 6, 6, 6, RESOL_Y - 6 - 1, RGB565(255, 255, 255), RESOL_Y - 12},
  { RESOL_X - 6 - 1, RESOL_X - 6 - 1, 6, RESOL_Y - 6 - 1, RGB565(255, 255, 255), RESOL_Y - 12},

  // RGB swatches
  { COLOR_STARTX1, COLOR_STARTX1 + COLOR_WIDTH - 1,
    COLOR_STARTY1, COLOR_STARTY1 + COLOR_HEIGHT - 1,
    RGB565(255, 0, 0), COLOR_WIDTH * COLOR_HEIGHT },
  { COLOR_STARTX1 + COLOR_DX * 1, COLOR_STARTX1 + COLOR_DX * 1 + COLOR_WIDTH - 1,
    COLOR_STARTY1 + COLOR_DY * 1, COLOR_STARTY1 + COLOR_DY * 1 + COLOR_HEIGHT - 1,
    RGB565(0, 255, 0), COLOR_WIDTH * COLOR_HEIGHT },
  { COLOR_STARTX1 + COLOR_DX * 2, COLOR_STARTX1 + COLOR_DX * 2 + COLOR_WIDTH - 1,
    COLOR_STARTY1 + COLOR_DY * 2, COLOR_STARTY1 + COLOR_DY * 2 + COLOR_HEIGHT - 1,
    RGB565(0, 0, 255), COLOR_WIDTH * COLOR_HEIGHT },
  { COLOR_STARTX1 + COLOR_DX * 3, COLOR_STARTX1 + COLOR_DX * 3 + COLOR_WIDTH - 1,
    COLOR_STARTY1 + COLOR_DY * 3, COLOR_STARTY1 + COLOR_DY * 3 + COLOR_HEIGHT - 1,
    RGB565(255, 255, 255), COLOR_WIDTH * COLOR_HEIGHT },

  // CMY swatches
  { COLOR_STARTX2, COLOR_STARTX2 + COLOR_WIDTH - 1,
    COLOR_STARTY2, COLOR_STARTY2 + COLOR_HEIGHT - 1,
    RGB565(0, 255, 255), COLOR_WIDTH * COLOR_HEIGHT },
  { COLOR_STARTX2 + COLOR_DX * 1, COLOR_STARTX2 + COLOR_DX * 1 + COLOR_WIDTH - 1,
    COLOR_STARTY2 + COLOR_DY * 1, COLOR_STARTY2 + COLOR_DY * 1 + COLOR_HEIGHT - 1,
    RGB565(255, 0, 255), COLOR_WIDTH * COLOR_HEIGHT },
  { COLOR_STARTX2 + COLOR_DX * 2, COLOR_STARTX2 + COLOR_DX * 2 + COLOR_WIDTH - 1,
    COLOR_STARTY2 + COLOR_DY * 2, COLOR_STARTY2 + COLOR_DY * 2 + COLOR_HEIGHT - 1,
    RGB565(255, 2550, 0), COLOR_WIDTH * COLOR_HEIGHT },
  { COLOR_STARTX2 + COLOR_DX * 3, COLOR_STARTX2 + COLOR_DX * 3 + COLOR_WIDTH - 1,
    COLOR_STARTY2 + COLOR_DY * 3, COLOR_STARTY2 + COLOR_DY * 3 + COLOR_HEIGHT - 1,
    RGB565(255, 255, 255), COLOR_WIDTH * COLOR_HEIGHT },

  // Grey swatches vertical 0%..100%
  //{ GREY_VSTARTX1, GREY_VSTARTX1 + GREY_VWIDTH - 1,
  //  GREY_VSTARTY1, GREY_VSTARTY1 + GREY_VHEIGHT - 1,
  //  RGB565(0, 0, 0), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 1, GREY_VSTARTX1 + GREY_VDX * 1 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 1, GREY_VSTARTY1 + GREY_VDY * 1 + GREY_VHEIGHT - 1,
    RGB565(25, 25, 25), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 2, GREY_VSTARTX1 + GREY_VDX * 2 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 2, GREY_VSTARTY1 + GREY_VDY * 2 + GREY_VHEIGHT - 1,
    RGB565(50, 50, 50), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 3, GREY_VSTARTX1 + GREY_VDX * 3 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 3, GREY_VSTARTY1 + GREY_VDY * 3 + GREY_VHEIGHT - 1,
    RGB565(75, 75, 75), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 4, GREY_VSTARTX1 + GREY_VDX * 4 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 4, GREY_VSTARTY1 + GREY_VDY * 4 + GREY_VHEIGHT - 1,
    RGB565(100, 100, 100), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 5, GREY_VSTARTX1 + GREY_VDX * 5 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 5, GREY_VSTARTY1 + GREY_VDY * 5 + GREY_VHEIGHT - 1,
    RGB565(125, 125, 125), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 6, GREY_VSTARTX1 + GREY_VDX * 6 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 6, GREY_VSTARTY1 + GREY_VDY * 6 + GREY_VHEIGHT - 1,
    RGB565(150, 150, 150), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 7, GREY_VSTARTX1 + GREY_VDX * 7 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 7, GREY_VSTARTY1 + GREY_VDY * 7 + GREY_VHEIGHT - 1,
    RGB565(175, 175, 175), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 8, GREY_VSTARTX1 + GREY_VDX * 8 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 8, GREY_VSTARTY1 + GREY_VDY * 8 + GREY_VHEIGHT - 1,
    RGB565(200, 200, 200), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 9, GREY_VSTARTX1 + GREY_VDX * 9 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 9, GREY_VSTARTY1 + GREY_VDY * 9 + GREY_VHEIGHT - 1,
    RGB565(225, 225, 225), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX1 + GREY_VDX * 10, GREY_VSTARTX1 + GREY_VDX * 10 + GREY_VWIDTH - 1,
    GREY_VSTARTY1 + GREY_VDY * 10, GREY_VSTARTY1 + GREY_VDY * 10 + GREY_VHEIGHT - 1,
    RGB565(255, 255, 255), GREY_VWIDTH * GREY_VHEIGHT },
  // Grey swatches vertical 100%..0%
  { GREY_VSTARTX2, GREY_VSTARTX2 + GREY_VWIDTH - 1,
    GREY_VSTARTY2, GREY_VSTARTY2 + GREY_VHEIGHT - 1,
    RGB565(255, 255, 255), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 1, GREY_VSTARTX2 + GREY_VDX * 1 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 1, GREY_VSTARTY2 + GREY_VDY * 1 + GREY_VHEIGHT - 1,
    RGB565(225, 225, 225), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 2, GREY_VSTARTX2 + GREY_VDX * 2 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 2, GREY_VSTARTY2 + GREY_VDY * 2 + GREY_VHEIGHT - 1,
    RGB565(200, 200, 200), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 3, GREY_VSTARTX2 + GREY_VDX * 3 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 3, GREY_VSTARTY2 + GREY_VDY * 3 + GREY_VHEIGHT - 1,
    RGB565(175, 175, 175), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 4, GREY_VSTARTX2 + GREY_VDX * 4 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 4, GREY_VSTARTY2 + GREY_VDY * 4 + GREY_VHEIGHT - 1,
    RGB565(150, 150, 150), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 5, GREY_VSTARTX2 + GREY_VDX * 5 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 5, GREY_VSTARTY2 + GREY_VDY * 5 + GREY_VHEIGHT - 1,
    RGB565(125, 125, 125), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 6, GREY_VSTARTX2 + GREY_VDX * 6 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 6, GREY_VSTARTY2 + GREY_VDY * 6 + GREY_VHEIGHT - 1,
    RGB565(100, 100, 100), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 7, GREY_VSTARTX2 + GREY_VDX * 7 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 7, GREY_VSTARTY2 + GREY_VDY * 7 + GREY_VHEIGHT - 1,
    RGB565(75, 75, 75), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 8, GREY_VSTARTX2 + GREY_VDX * 8 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 8, GREY_VSTARTY2 + GREY_VDY * 8 + GREY_VHEIGHT - 1,
    RGB565(50, 50, 50), GREY_VWIDTH * GREY_VHEIGHT },
  { GREY_VSTARTX2 + GREY_VDX * 9, GREY_VSTARTX2 + GREY_VDX * 9 + GREY_VWIDTH - 1,
    GREY_VSTARTY2 + GREY_VDY * 9, GREY_VSTARTY2 + GREY_VDY * 9 + GREY_VHEIGHT - 1,
    RGB565(25, 25, 25), GREY_VWIDTH * GREY_VHEIGHT },
  //{ GREY_VSTARTX2 + GREY_VDX * 10, GREY_VSTARTX2 + GREY_VDX * 10 + GREY_VWIDTH - 1,
  //  GREY_VSTARTY2 + GREY_VDY * 10, GREY_VSTARTY2 + GREY_VDY * 10 + GREY_VHEIGHT - 1,
  //  RGB565(0, 0, 0), GREY_VWIDTH * GREY_VHEIGHT },

#ifdef GREY_HSTARTX1
  // Grey swatches horizontal 0%..100%
  //{ GREY_HSTARTX1, GREY_HSTARTX1 + GREY_HWIDTH - 1,
  //  GREY_HSTARTY1, GREY_HSTARTY1 + GREY_HHEIGHT - 1,
  //  RGB565(0, 0, 0), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 1, GREY_HSTARTX1 + GREY_HDX * 1 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 1, GREY_HSTARTY1 + GREY_HDY * 1 + GREY_HHEIGHT - 1,
    RGB565(25, 25, 25), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 2, GREY_HSTARTX1 + GREY_HDX * 2 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 2, GREY_HSTARTY1 + GREY_HDY * 2 + GREY_HHEIGHT - 1,
    RGB565(50, 50, 50), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 3, GREY_HSTARTX1 + GREY_HDX * 3 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 3, GREY_HSTARTY1 + GREY_HDY * 3 + GREY_HHEIGHT - 1,
    RGB565(75, 75, 75), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 4, GREY_HSTARTX1 + GREY_HDX * 4 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 4, GREY_HSTARTY1 + GREY_HDY * 4 + GREY_HHEIGHT - 1,
    RGB565(100, 100, 100), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 5, GREY_HSTARTX1 + GREY_HDX * 5 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 5, GREY_HSTARTY1 + GREY_HDY * 5 + GREY_HHEIGHT - 1,
    RGB565(125, 125, 125), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 6, GREY_HSTARTX1 + GREY_HDX * 6 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 6, GREY_HSTARTY1 + GREY_HDY * 6 + GREY_HHEIGHT - 1,
    RGB565(150, 150, 150), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 7, GREY_HSTARTX1 + GREY_HDX * 7 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 7, GREY_HSTARTY1 + GREY_HDY * 7 + GREY_HHEIGHT - 1,
    RGB565(175, 175, 175), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 8, GREY_HSTARTX1 + GREY_HDX * 8 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 8, GREY_HSTARTY1 + GREY_HDY * 8 + GREY_HHEIGHT - 1,
    RGB565(200, 200, 200), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 9, GREY_HSTARTX1 + GREY_HDX * 9 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 9, GREY_HSTARTY1 + GREY_HDY * 9 + GREY_HHEIGHT - 1,
    RGB565(225, 225, 225), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX1 + GREY_HDX * 10, GREY_HSTARTX1 + GREY_HDX * 10 + GREY_HWIDTH - 1,
    GREY_HSTARTY1 + GREY_HDY * 10, GREY_HSTARTY1 + GREY_HDY * 10 + GREY_HHEIGHT - 1,
    RGB565(255, 255, 255), GREY_HWIDTH * GREY_HHEIGHT },
  // Grey swatches horizontal 100%..0%
  { GREY_HSTARTX2, GREY_HSTARTX2 + GREY_HWIDTH - 1,
    GREY_HSTARTY2, GREY_HSTARTY2 + GREY_HHEIGHT - 1,
    RGB565(255, 255, 255), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 1, GREY_HSTARTX2 + GREY_HDX * 1 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 1, GREY_HSTARTY2 + GREY_HDY * 1 + GREY_HHEIGHT - 1,
    RGB565(225, 225, 225), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 2, GREY_HSTARTX2 + GREY_HDX * 2 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 2, GREY_HSTARTY2 + GREY_HDY * 2 + GREY_HHEIGHT - 1,
    RGB565(200, 200, 200), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 3, GREY_HSTARTX2 + GREY_HDX * 3 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 3, GREY_HSTARTY2 + GREY_HDY * 3 + GREY_HHEIGHT - 1,
    RGB565(175, 175, 175), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 4, GREY_HSTARTX2 + GREY_HDX * 4 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 4, GREY_HSTARTY2 + GREY_HDY * 4 + GREY_HHEIGHT - 1,
    RGB565(150, 150, 150), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 5, GREY_HSTARTX2 + GREY_HDX * 5 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 5, GREY_HSTARTY2 + GREY_HDY * 5 + GREY_HHEIGHT - 1,
    RGB565(125, 125, 125), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 6, GREY_HSTARTX2 + GREY_HDX * 6 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 6, GREY_HSTARTY2 + GREY_HDY * 6 + GREY_HHEIGHT - 1,
    RGB565(100, 100, 100), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 7, GREY_HSTARTX2 + GREY_HDX * 7 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 7, GREY_HSTARTY2 + GREY_HDY * 7 + GREY_HHEIGHT - 1,
    RGB565(75, 75, 75), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 8, GREY_HSTARTX2 + GREY_HDX * 8 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 8, GREY_HSTARTY2 + GREY_HDY * 8 + GREY_HHEIGHT - 1,
    RGB565(50, 50, 50), GREY_HWIDTH * GREY_HHEIGHT },
  { GREY_HSTARTX2 + GREY_HDX * 9, GREY_HSTARTX2 + GREY_HDX * 9 + GREY_HWIDTH - 1,
    GREY_HSTARTY2 + GREY_HDY * 9, GREY_HSTARTY2 + GREY_HDY * 9 + GREY_HHEIGHT - 1,
    RGB565(25, 25, 25), GREY_HWIDTH * GREY_HHEIGHT },
  //{ GREY_HSTARTX2 + GREY_HDX * 10, GREY_HSTARTX2 + GREY_HDX * 10 + GREY_HWIDTH - 1,
  //  GREY_HSTARTY2 + GREY_HDY * 10, GREY_HSTARTY2 + GREY_HDY * 10 + GREY_HHEIGHT - 1,
  //  RGB565(0, 0, 0), GREY_HWIDTH * GREY_HHEIGHT },
#endif
};
	
//#define SETRECT(x, y, w, h) g_blit.x0 = x; g_blit.x1 = x + w - 1; g_blit.y0 = y; g_blit.y1 = y + h -1; disp_blit();

static void fill(short rgb565, unsigned int n)
{
	rgb565;
	while (n--)
	{
		_asm
		.include "dpf.inc"
		
		mov	a, dph
		lcall lcd_write
		mov	a, dpl
		lcall lcd_write
		_endasm;
	}
}

static void show_testpattern()
{
	BYTE i;
	const __code SWATCH *p;
	
	clrscreen(COLOR_BLACK);

	for (i = 0; i < sizeof(swatches) / sizeof(SWATCH); i++)
	{
		p = swatches+i;
		g_blit.x0 = p->xs;
		g_blit.x1 = p->xe;
		g_blit.y0 = p->ys;
		g_blit.y1 = p->ye;
		disp_blit();
		fill(p->color, p->len);
	}
#if 0
	SETRECT(10,  20, 80, 80); fill(RGB565(255, 0, 0), 80 * 80);
	SETRECT(10, 101, 80, 80); fill(RGB565(0, 255, 0), 80 * 80);
	SETRECT(10, 182, 80, 80); fill(RGB565(0, 0, 255), 80 * 80);

	SETRECT(100,  20, 40, 25); fill(RGB565(  0,   0,   0), 40 * 25);
	SETRECT(100,  45, 40, 25); fill(RGB565( 25,  25,  25), 40 * 25);
	SETRECT(100,  70, 40, 25); fill(RGB565( 50,  50,  50), 40 * 25);
	SETRECT(100,  95, 40, 25); fill(RGB565( 75,  75,  75), 40 * 25);
	SETRECT(100,  120, 40, 25); fill(RGB565( 100, 100, 100), 40 * 25);
	SETRECT(100,  145, 40, 25); fill(RGB565( 125, 125, 125), 40 * 25);
	SETRECT(100,  170, 40, 25); fill(RGB565(150, 150, 150), 40 * 25);
	SETRECT(100,  195, 40, 25); fill(RGB565(200, 200, 200), 40 * 25);
	SETRECT(100,  220, 40, 25); fill(RGB565(225, 225, 225), 40 * 25);
	SETRECT(100,  245, 40, 25); fill(RGB565(255, 255, 255), 40 * 25);

	SETRECT(140,  245, 40, 25); fill(RGB565(  0,   0,   0), 40 * 25);
	SETRECT(140,  220, 40, 25); fill(RGB565( 25,  25,  25), 40 * 25);
	SETRECT(140,  195, 40, 25); fill(RGB565( 50,  50,  50), 40 * 25);
	SETRECT(140,  170, 40, 25); fill(RGB565( 75,  75,  75), 40 * 25);
	SETRECT(140,  145, 40, 25); fill(RGB565( 100, 100, 100), 40 * 25);
	SETRECT(140,  120, 40, 25); fill(RGB565( 125, 125, 125), 40 * 25);
	SETRECT(140,  95, 40, 25); fill(RGB565(150, 150, 150), 40 * 25);
	SETRECT(140,  70, 40, 25); fill(RGB565(200, 200, 200), 40 * 25);
	SETRECT(140,  45, 40, 25); fill(RGB565(225, 225, 225), 40 * 25);
	SETRECT(140,  20, 40, 25); fill(RGB565(255, 255, 255), 40 * 25);
#endif
}


